#!/bin/bash
#
# xnviewmp-install:xnviewmp-install.sh
#
# Install XnViewMP on Mint (should work on any Debian based linux)
#
set -u
set -o pipefail

declare -gr DEFAULT_DEB_FILES_INSTALL_COMMON_HOME='/usr/local/share/debfiles'
declare -gr DEFAULT_DEB_FILES_INSTALL_XNVIEWMP_HOME="${DEFAULT_DEB_FILES_INSTALL_COMMON_HOME}/xnviewmp"

declare -gr DEFAULT_DEB_FILES_INSTALL_XNVIEWMP_VERSION="${DEFAULT_DEB_FILES_INSTALL_XNVIEWMP_HOME}/xnviewmp.version"

function usage {
  cat <<EOF
Install XnViewMP on Mint for Debian based linux.

Usage:

    $( basename "$0" ) --help
    $( basename "$0" ) --list
    $( basename "$0" ) --list-fullname
    $( basename "$0" ) --install [--home DIR] [--version VER|--last-default-version]
    $( basename "$0" ) --local-packages [--home DIR]

Actions:

  -h,   --help      Display this message and exit.
  -l,   --list      List version numbers available on server
  -L,   --list-fullname
                    List of all XNViewMP versions available on server
  -i,   --install   Download and install XNViewMP for all users.
                    If --version not use, compute last version available for your arch, then
                    install computed version.
                    If you dont run command using root, then sudo will be use.
        --local-packages
                    Show local packages
  -C,   --cleanup-packages
                    Allow to cleanup local- packages previous installed packages

Options:

           --debug  Show more informations to understand remote site changes.
           --home DIR
                    Set download directory
                    (default value: '$( compute_DEFAULT_DEB_FILES_INSTALL_HOME )')
  -V VER,  --version VER
                    Download and install given version (ie: '090')
  -L0,     --last-default-version
                    Use version from XNView home page (this version did have a version number - not recommended).

EOF
}

#
# P_param : get parameter value, if not exist call usage and return error.
#
# Usage:
#   param_xxx="$( P_param "$@" )" && shift || return $?
#
# Require:
#   - param variable with current parameter name
#   - usage function (use to display usage)
#
function P_param {
  if [ "$#" -gt 0 ] ; then
    echo "$1"
  else
    echofatal "Value required for '${param}'"
    usage >&2
    return 1
  fi
}

function echodebug {
  echo '* [DEBUG]: ' "$@" >&2
}

function echofatal {
  echo '*** [FATAL]: ' "$@" >&2
}

function echoinfo {
  echo '* [INFO]: ' "$@" >&2
}

function echookay {
  echo '* [OK]: ' "$@" >&2
}

function echowarn {
  echo '* [WARN]: ' "$@" >&2
}

function check_if_sudo_required {
  if [[ $(id -u) -ne 0 ]] ; then
    SUDO_IF_NEEDED='sudo'
  else
    SUDO_IF_NEEDED=''
  fi
}

function cleanup_local_packages {
  check_if_sudo_required

  local deb_files=
  deb_files="$( get_local_packages )" || return $?
  deb_files="$( sort -r <<<"${deb_files}" | tail -n +2 )" || return $?

  echo "${deb_files}" | "${SUDO_IF_NEEDED}" xargs --open-tty rm -i -v || return $?
}

function compute_DEFAULT_DEB_FILES_INSTALL_HOME {
  echo "${DEFAULT_DEB_FILES_INSTALL_XNVIEWMP_HOME}"
}

function get_DEB_FILES_INSTALL_HOME {
  if [ -n "${param_download_directory}" ] ; then
    echo "${param_download_directory}"
  else
    compute_DEFAULT_DEB_FILES_INSTALL_HOME || return $?
  fi
}

function get_arch {
  local arch=
  arch="$( uname -m )" || return $?

  case "${arch}" in
    'i'?'86')
      echo 'i386'
      ;;

    'x86_64'|'ppc64')
      echo "${arch}"
      ;;

    *)
      echo "${arch}"
      ;;
  esac
}

function get_suffix_arch_for_xnview {
  local arch=
  arch="$( get_arch )" || return $?

  case "${arch}" in
    'i386')
      echo ''
      ;;

    'x86_64')
      echo '-x64'
      ;;

    *)
      echofatal "Unsupported plateform: '${arch}'"
      return 100
      ;;
  esac
}

function build_output_filename {
  local -r prefix="$1"
  local -r suffix="$2"

# echo "${prefix}XnViewMP-${suffix}.deb"
  echo "${prefix}XnView_MP-${suffix}.deb"
}

function __download_xnviewmp_unsafe__ {
  local -r _url_="$1"
  local -r _filename_="$2"

  (
    set -x
    ${SUDO_IF_NEEDED} wget "${_url_}" --output-document "${_filename_}" >&2
  ) || return $?
}

function download_xnviewmp {
  local -r url="$1"
  local -r filename="$2"

  echoinfo "Download URL: '${url}'"
  echoinfo "Downloaded FILE: '$( realpath "${filename}" )'"

  if __download_xnviewmp_unsafe__ "${url}" "${filename}.tmp" ; then
    (
      set -x
      ${SUDO_IF_NEEDED} mv -f "${filename}.tmp" "${filename}"
    ) || return $?
  else
    (
      set -x
      ${SUDO_IF_NEEDED} rm -f "${filename}.tmp"
    ) || return $?
    (
      set -x
      curl "${curl_params[@]}" --url "${url}" --head >&2
    )
    return 100
  fi
}

function get_last_xnviewmp_url {
  local -r nxview_arch="$1"

  echo "https://download.xnview.com/XnViewMP-linux${nxview_arch}.deb"
}

function get_local_packages {
  local dir=
  dir="$( get_DEB_FILES_INSTALL_HOME )" || return $?

  echoinfo "${FUNCNAME[0]}() - Directory: '${dir}'"

  find "${dir}" -mindepth 1 -maxdepth 1 -type f -name '*.deb' | sort || return $?
}

#
# https://download.xnview.com/old_versions/
#
function get_specific_xnviewmp_version {
  local -r fullversion="$1" # 090-linux-x64

# get_specific_xnviewmp_file "XnViewMP-${fullversion}.deb"
  get_specific_xnviewmp_file "XnView_MP-${fullversion}.deb"
}

function get_specific_xnviewmp_file {
  local -r filename="$1" # XnViewMP-090-linux-x64.deb

  ##### https://download.xnview.com/old_versions/XnViewMP-090-linux-x64.deb
  #cho "https://download.xnview.com/old_versions/${filename}"
  ##### https://download.xnview.com/old_versions/XnView_MP/XnView_MP-1.3.1-linux-x64.deb
  echo "https://download.xnview.com/old_versions/XnView_MP/${filename}"
}

function remove_xnview {
  echoinfo 'Remove debian package (dpkg)'
  (
    ${SUDO_IF_NEEDED} dpkg -r xnview
  ) || true
  echoinfo 'Remove debian package (dpkg) : DONE'

  echoinfo 'Remove std package (apt)'
  (
    ${SUDO_IF_NEEDED} apt remove xnview
  ) || true
  echoinfo 'Remove std package (apt) : DONE'
}

function update_or_install_nxview_if_needed {
  local -r url="$1"
  local -r download_directory="$2"
  local -r local_filename="$3"

  if [ "${DEBUG:-}" = 'true' ] ; then
    echodebug "${FUNCNAME[0]}() - url='${url}'"
    echodebug "${FUNCNAME[0]}() - download_directory='${download_directory}'"
    echodebug "${FUNCNAME[0]}() - local_filename='${local_filename}'"
  fi

  if [ ! -d "${download_directory}" ] ; then
    (
      ${SUDO_IF_NEEDED} mkdir -p "${download_directory}"
    ) || return $?
  fi

  pushd "${download_directory}" >/dev/null || return $?

  if [ -f "${local_filename}" ] ; then
    echowarn "File '$( realpath "${local_filename}" )' already present, skip download." || return 100
  else
    download_xnviewmp "${url}" "${local_filename}" || return $?
  fi

  install_nxview_if_needed || return $?

  popd >/dev/null || return $?
}

function install_nxview_if_needed {
  if [ -f "${DEFAULT_DEB_FILES_INSTALL_XNVIEWMP_VERSION}" ] ; then
    local installed_version=
    read -r installed_version <"${DEFAULT_DEB_FILES_INSTALL_XNVIEWMP_VERSION}"

    echoinfo "Installed version is: '${installed_version}'"
    if [ "${installed_version}" = "${local_filename}" ] ; then
      echoinfo 'You already have last version'
      return 0
    fi
  fi

  remove_xnview || true

  (
    ${SUDO_IF_NEEDED} dpkg -i "${local_filename}"
  ) || return $?
  (
    echo "${local_filename}" | ${SUDO_IF_NEEDED} tee "${DEFAULT_DEB_FILES_INSTALL_XNVIEWMP_VERSION}"
  ) || return $?
  (
    ${SUDO_IF_NEEDED} chmod 0644 "${DEFAULT_DEB_FILES_INSTALL_XNVIEWMP_VERSION}"
  ) || return $?
}

function list_all_app_versions {
  local -a curl_params=()
  if [ "${DEBUG:-}" != 'true' ] ; then
    curl_params+=(--silent)
  fi
  curl_params+=(--show-error)

  local _page_content_=
  _page_content_="$(
    set -x
    curl "${curl_params[@]}" 'https://download.xnview.com/old_versions/XnView_MP/'
    )" || return $?

  if [ "${DEBUG:-}" = 'true' ] ; then
    cat <<EOF >&2
---PAGE
${_page_content_}
---
EOF
  fi

  # Keep Debian packages only and sort
  _page_content_="$(
    grep -E '"XnView_MP-(.*)\.deb"' <<<"${_page_content_}" |
    sed -E 's,^.*[[:blank:]]href="XnView_MP-(.*)\.deb".*$,XnView_MP-\1.deb,g' |
    sort
    )"

  if [ "${DEBUG:-}" = 'true' ] ; then
    cat <<EOF >&2
---FILTER
${_page_content_}
---
EOF
  fi

  local line=
  while read -r line ; do
    if [ -n "${line}" ] ; then
      if [ "${DEBUG:-}" = 'true' ] ; then
        echodebug "FIND: '${line}'"
      fi
      echo "${line}"
    fi
  done <<<"${_page_content_}"
}

function list_versions_only_raw {
  local _versions_=
  _versions_="$( list_all_app_versions )" || return $?

  local filter=
  case "$( get_arch )" in
    'i386')
      filter='-linux.deb$'
      ;;

    'x86_64')
      filter='-linux-x64.deb$'
      ;;

    *)
      echofatal "Unsupported plateform: '${arch}'"
      return 100
      ;;
  esac

  # Get version number only for arch
  grep -- "${filter}" <<<"${_versions_}" | sed -E "s,^XnView_MP-(.*)${filter},\1,g"
}

function list_versions_only {
  list_versions_only_raw | sort | uniq || return $?
}

function download_and_install_if_needed {
  local nxview_arch_suffix=
  nxview_arch_suffix="$( get_suffix_arch_for_xnview )" || return $?
  local -r nxview_arch_suffix

  if [ "${DEBUG:-}" = 'true' ] ; then
    echodebug "${FUNCNAME[0]}() - param_nxview_version='${param_nxview_version}'"
    echodebug "${FUNCNAME[0]}() - nxview_arch_suffix='${nxview_arch_suffix}'"
  fi

  local filename=
  local url=
  case "${param_nxview_version}" in
    'currentlast')
      local tag= # no version in filename, use download date to have new name (so file will be downloaded and install at every run).

      tag="$( date +'%Y-%m-%d_%H-%M' )" || return $?
      filename="$( build_output_filename "${tag}." "linux${nxview_arch_suffix}" )" || return $?
      url="$( get_last_xnviewmp_url "${nxview_arch_suffix}" )"
      ;;

    'latest') # DEFAULT
      filename="$( list_all_app_versions | grep "linux${nxview_arch_suffix}.deb" | tail -n 1 )" || exit 1
      url="$( get_specific_xnviewmp_file "${filename}" )"
      ;;

    *)
      local -r fullversion="${param_nxview_version}-linux${nxview_arch_suffix}" # 090-linux-x64

      filename="$( build_output_filename '' "${fullversion}" )" || exit 1
      url="$( get_specific_xnviewmp_version "${fullversion}" )"
      ;;
  esac

  local download_directory=
  download_directory="$( get_DEB_FILES_INSTALL_HOME )" || return $?

  update_or_install_nxview_if_needed \
        "${url}" \
        "${download_directory}" \
        "${filename}" || return $?
}

function main {
  local action='--install'
  local param=
  local param_download_directory=
  local param_nxview_version='latest'

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        return 0
        ;;

      '-l'|'--list'             ) action='--list' ;;
      '-L'|'--list-fullname'    ) action='--list-fullname' ;;
      '-i'|'--install'          ) action='--install' ;;
      '--local-packages'        ) action='--local-packages' ;;
      '-C'|'--cleanup-packages' ) action='--cleanup-packages' ;;

      '--home')
        param_download_directory="$( P_param "$@" )" && shift || return $?
        ;;

      '-V'|'--version')
        param_nxview_version="$( P_param "$@" )" && shift || return $?
        ;;

      '-L0'|'--last-default-version')
        param_nxview_version='currentlast'
        ;;

      '--debug')
        DEBUG='true'
        ;;

      *)
        echofatal "Unknown option: '${param}'."
        usage >&2
        return 1
        ;;
    esac
  done

  case "${action}" in
    '--list')
      local versions=
      versions="$( list_versions_only )" || return $?
      echo "${versions}"
      ;;

    '--list-fullname')
      local versions=
      versions="$( list_all_app_versions )" || return $?
      echo "${versions}"
      ;;

    '--local-packages')
      get_local_packages || return $?
      ;;

    '--cleanup-packages')
      cleanup_local_packages || return $?
      ;;

    '--install')
      check_if_sudo_required || return $?
      download_and_install_if_needed || return $?
      echoinfo 'Install done, try running xnviewmp from command line: xnview'
      ;;

    *)
      echofatal "Internal error action='${action}'"
      return 1
  esac
  echookay "$( basename "$0" ) - Done"
}

main "$@" || exit $?
